#!/usr/local/bin/python
#
# $Id: //projects/zoe/chaos.py#2 $ $Date: 2002/08/09 $

"""
A demonstration of chaos theory (namely sensitivity to initial
conditions) with a Lorenz system.
"""

import zoe


# The Lorenz differential equations:
#
#     dx/dt = sigma (y - x)
#     dy/dt = rho x - y - x z
#     dz/dt = -B z + x y

PARTICLES = 10 # the number of particles


class LorenzParticle(zoe.Particle):

    """A Lorenz particle simply moves according to the Lorenz
    differential equations described above."""

    SIGMA = 10.0
    RHO = 28.0
    B = 8.0/3.0
    DELTA_T = 0.01
    
    trailLength = 100

    def __init__(self, start):
        zoe.Particle.__init__(self, start)
        self.t = 0.0

    def update(self):
        x, y, z = self.pos
        zoe.Particle.update(self)
        deltaX = (self.SIGMA*(y - x))*self.DELTA_T
        deltaY = (self.RHO*x - y - x*z)*self.DELTA_T
        deltaZ = (-self.B*z + x*y)*self.DELTA_T
        x += deltaX
        y += deltaY
        z += deltaZ
        self.pos = x, y, z
        self.t += self.DELTA_T


class LorenzGroup(zoe.Group):

    """A Lorenz group creates a collection of particles that all have
    very nearly the same position, differing only by a very small
    amount."""

    BASE = 0.001 # the starting position
    VARIANCE = 0.000001 # the variance of all particles

    def __init__(self, count):
        zoe.Group.__init__(self)
        for i in range(1, count + 1):
            xyz = self.BASE + i*self.VARIANCE/count
            self.objects.append(LorenzParticle((xyz,)*3))


def main():
    engine = zoe.Engine()
    engine.setup()
    engine.camera.zoomOut(10.0)
    engine.add(zoe.AxesObject(50.0))
    engine.add(LorenzGroup(PARTICLES))
    engine.add(zoe.FrameRateCounter(engine))
    engine.go()

if __name__ == '__main__': main()
